import L from "leaflet";
import './L.Renderer.SVG.Tile.js';
import { PointSymbolizer } from './Symbolizer.Point.js';
import { LineSymbolizer } from './Symbolizer.Line.js';
import { FillSymbolizer } from './Symbolizer.Fill.js';
L.VectorGrid = L.GridLayer.extend({
    options: {
        rendererFactory: L.svg.tile,
        vectorTileLayerStyles: {},
        interactive: false,
    },
    initialize: function (options) {
        L.setOptions(this, options);
        L.GridLayer.prototype.initialize.apply(this, arguments);
        if (this.options.getFeatureId) {
            this._vectorTiles = {};
            this._overriddenStyles = {};
            this.on('tileunload', function (e) {
                var key = this._tileCoordsToKey(e.coords), tile = this._vectorTiles[key];
                if (tile && this._map) {
                    tile.removeFrom(this._map);
                }
                delete this._vectorTiles[key];
            }, this);
        }
        this._dataLayerNames = {};
    },
    createTile: function (coords, done) {
        var storeFeatures = this.options.getFeatureId;
        var tileSize = this.getTileSize();
        var renderer = this.options.rendererFactory(coords, tileSize, this.options);
        var vectorTilePromise = this._getVectorTilePromise(coords);
        if (storeFeatures) {
            this._vectorTiles[this._tileCoordsToKey(coords)] = renderer;
            renderer._features = {};
        }
        vectorTilePromise.then(function renderTile(vectorTile) {
            for (var layerName in vectorTile.layers) {
                this._dataLayerNames[layerName] = true;
                var layer = vectorTile.layers[layerName];
                var pxPerExtent = this.getTileSize().divideBy(layer.extent);
                var layerStyle = this.options.vectorTileLayerStyles[layerName] ||
                    L.Path.prototype.options;
                for (var i = 0; i < layer.features.length; i++) {
                    var feat = layer.features[i];
                    var id = void 0;
                    var styleOptions = layerStyle;
                    if (storeFeatures) {
                        id = this.options.getFeatureId(feat);
                        var styleOverride = this._overriddenStyles[id];
                        if (styleOverride) {
                            if (styleOverride[layerName]) {
                                styleOptions = styleOverride[layerName];
                            }
                            else {
                                styleOptions = styleOverride;
                            }
                        }
                    }
                    if (styleOptions instanceof Function) {
                        styleOptions = styleOptions(feat.properties, coords.z);
                    }
                    if (!(styleOptions instanceof Array)) {
                        styleOptions = [styleOptions];
                    }
                    if (!styleOptions.length) {
                        continue;
                    }
                    var featureLayer = this._createLayer(feat, pxPerExtent);
                    if (!featureLayer) {
                        continue;
                    }
                    for (var j = 0; j < styleOptions.length; j++) {
                        var style = L.extend({}, L.Path.prototype.options, styleOptions[j]);
                        featureLayer.render(renderer, style);
                        renderer._addPath(featureLayer);
                    }
                    if (this.options.interactive) {
                        featureLayer.makeInteractive();
                    }
                    if (storeFeatures) {
                        renderer._features[id] = {
                            layerName: layerName,
                            feature: featureLayer
                        };
                    }
                }
            }
            if (this._map != null) {
                renderer.addTo(this._map);
            }
            L.Util.requestAnimFrame(done.bind(coords, null, null));
        }.bind(this));
        return renderer.getContainer();
    },
    setFeatureStyle: function (id, layerStyle) {
        this._overriddenStyles[id] = layerStyle;
        for (var tileKey in this._vectorTiles) {
            var tile = this._vectorTiles[tileKey];
            var features = tile._features;
            var data = features[id];
            if (data) {
                var feat = data.feature;
                var styleOptions = layerStyle;
                if (layerStyle[data.layerName]) {
                    styleOptions = layerStyle[data.layerName];
                }
                this._updateStyles(feat, tile, styleOptions);
            }
        }
        return this;
    },
    resetFeatureStyle: function (id) {
        delete this._overriddenStyles[id];
        for (var tileKey in this._vectorTiles) {
            var tile = this._vectorTiles[tileKey];
            var features = tile._features;
            var data = features[id];
            if (data) {
                var feat = data.feature;
                var styleOptions = this.options.vectorTileLayerStyles[data.layerName] ||
                    L.Path.prototype.options;
                this._updateStyles(feat, tile, styleOptions);
            }
        }
        return this;
    },
    getDataLayerNames: function () {
        return Object.keys(this._dataLayerNames);
    },
    _updateStyles: function (feat, renderer, styleOptions) {
        styleOptions = (styleOptions instanceof Function) ?
            styleOptions(feat.properties, renderer.getCoord().z) :
            styleOptions;
        if (!(styleOptions instanceof Array)) {
            styleOptions = [styleOptions];
        }
        for (var j = 0; j < styleOptions.length; j++) {
            var style = L.extend({}, L.Path.prototype.options, styleOptions[j]);
            feat.updateStyle(renderer, style);
        }
    },
    _createLayer: function (feat, pxPerExtent, layerStyle) {
        var layer;
        switch (feat.type) {
            case 1:
                layer = new PointSymbolizer(feat, pxPerExtent);
                layer.getLatLng = null;
                break;
            case 2:
                layer = new LineSymbolizer(feat, pxPerExtent);
                break;
            case 3:
                layer = new FillSymbolizer(feat, pxPerExtent);
                break;
        }
        if (this.options.interactive) {
            try {
                layer.addEventParent(this);
            }
            catch (error) {
                return null;
            }
        }
        return layer;
    },
});
L.vectorGrid = function (options) {
    return new L.VectorGrid(options);
};
export default L.vectorGrid;
