import L from "leaflet";
import WebMecatorTileUtils from "./WebMecatorTileUtils";
var OgcXmlParser = (function () {
    function OgcXmlParser() {
    }
    OgcXmlParser.getWmtsCapabilitiesUrl = function (url) {
        if (url.indexOf("?") !== -1) {
            return url + "&request=GetCapabilities&service=wmts";
        }
        else {
            return url + "?request=GetCapabilities&service=wmts";
        }
    };
    OgcXmlParser.getWmsCapabilitiesUrl = function (url, version) {
        if (version === void 0) { version = "1.3.0"; }
        if (url.indexOf("?") !== -1) {
            return url + "&SERVICE=WMS&VERSION=".concat(version, "&REQUEST=GetCapabilities");
        }
        else {
            return url + "?SERVICE=WMS&VERSION=".concat(version, "&REQUEST=GetCapabilities");
        }
    };
    OgcXmlParser.getWmsLayers = function (xmlData) {
        var layerNames = [];
        var domParser = new DOMParser();
        var xmlDoc = domParser.parseFromString(xmlData, "text/xml");
        var names = xmlDoc.getElementsByTagName("Layer");
        var len = names.length;
        for (var i = 1; i < len; i++) {
            var layer = names[i];
            var nodes = layer.childNodes;
            for (var j = 0; j < nodes.length; j++) {
                var eleName = nodes[j].nodeName;
                if ("Name" === eleName) {
                    layerNames.push(nodes[j].textContent);
                }
            }
        }
        return layerNames.join();
    };
    OgcXmlParser.getWmtsLayer = function (xmlData) {
        var domParser = new DOMParser();
        var xmlDoc = domParser.parseFromString(xmlData, "text/xml");
        var childNodes = xmlDoc.getElementsByTagName("Layer")[0].childNodes;
        for (var index = 0; index < childNodes.length; index++) {
            var ele = childNodes[index];
            var eleName = ele.nodeName;
            if ("ows:Identifier" === eleName) {
                return ele.textContent;
            }
        }
    };
    OgcXmlParser.getWmtsLayerTitle = function (xmlData) {
        var domParser = new DOMParser();
        var xmlDoc = domParser.parseFromString(xmlData, "text/xml");
        var childNodes = xmlDoc.getElementsByTagName("Layer")[0].childNodes;
        for (var index = 0; index < childNodes.length; index++) {
            var ele = childNodes[index];
            var eleName = ele.nodeName;
            if ("ows:Title" === eleName) {
                return ele.textContent;
            }
        }
    };
    OgcXmlParser.getWmtsTileMatrix = function (xmlData) {
        var domParser = new DOMParser();
        var xmlDoc = domParser.parseFromString(xmlData, "text/xml");
        var childNodes = xmlDoc.getElementsByTagName("Layer")[0].childNodes;
        for (var index = 0; index < childNodes.length; index++) {
            var ele = childNodes[index];
            var eleName = ele.nodeName;
            if ("TileMatrixSetLink" === eleName) {
                var tileMatrixSet = ele.childNodes[1];
                if ("TileMatrixSet" === tileMatrixSet.nodeName) {
                    return tileMatrixSet.textContent;
                }
            }
        }
    };
    OgcXmlParser.getWmtsOptions = function (xmlData) {
        var title = this.getWmtsLayerTitle(xmlData);
        var layer = this.getWmtsLayer(xmlData);
        var tilematrixset = this.getWmtsTileMatrix(xmlData);
        var center = this.getOgcCenter(xmlData);
        var bounds = this.getFitBounds(xmlData);
        return {
            title: title,
            layer: layer,
            tilematrixset: tilematrixset,
            center: center,
            bounds: bounds,
        };
    };
    OgcXmlParser.getWmsOptions = function (xmlData, options) {
        if (options === void 0) { options = { format: "image/png", transparent: true }; }
        var layers = this.getWmsLayers(xmlData);
        var center = this.getOgcCenter(xmlData, OgcXmlParser.WMS);
        var bounds = this.getFitBounds(xmlData, OgcXmlParser.WMS);
        return Object.assign({
            layers: "".concat(layers),
            center: center,
            bounds: bounds,
        }, options);
    };
    OgcXmlParser.getWmsBounds = function (xmlData) {
        var domParser = new DOMParser();
        var xmlDoc = domParser.parseFromString(xmlData, "text/xml");
        var names = xmlDoc.getElementsByTagName("Layer");
        var len = names.length;
        for (var i = 1; i < len; i++) {
            var layer = names[i];
            var nodes = layer.childNodes;
            for (var j = 0; j < nodes.length; j++) {
                var eleName = nodes[j].nodeName;
                if ("EX_GeographicBoundingBox" === eleName) {
                    var node = nodes[j];
                    var child = node.childNodes;
                    var west = child[0].textContent;
                    var east = child[1].textContent;
                    var south = child[2].textContent;
                    var north = child[3].textContent;
                    var bounds = {
                        minLat: Number(south),
                        minLon: Number(west),
                        maxLat: Number(north),
                        maxLon: Number(east),
                    };
                    return this.wrapperBounds(bounds);
                }
            }
        }
    };
    OgcXmlParser.wrapperBounds = function (bounds) {
        if (bounds.minLat < -90 ||
            bounds.maxLat > 90 ||
            bounds.minLon < -180 ||
            bounds.maxLon > 180) {
            var minLatLon = WebMecatorTileUtils.meters2LatLon(bounds.minLat, bounds.minLon);
            var maxLatLon = WebMecatorTileUtils.meters2LatLon(bounds.maxLat, bounds.maxLon);
            return {
                minLat: minLatLon.lat,
                maxLat: maxLatLon.lat,
                minLon: minLatLon.lon,
                maxLon: maxLatLon.lon,
            };
        }
        else {
            return bounds;
        }
    };
    OgcXmlParser.getFitBounds = function (xmlData, service) {
        if (service === void 0) { service = OgcXmlParser.WMTS; }
        var bounds = {};
        if (OgcXmlParser.WMTS === service) {
            bounds = this.getWMTSBounds(xmlData);
        }
        else if (OgcXmlParser.WMS === service) {
            bounds = this.getWmsBounds(xmlData);
        }
        var minLatLng = this.createCoordinate(bounds.minLat, bounds.minLon);
        var maxLatLng = this.createCoordinate(bounds.maxLat, bounds.maxLon);
        return L.latLngBounds(minLatLng, maxLatLng);
    };
    OgcXmlParser.getWMTSBounds = function (xmlData) {
        var domParser = new DOMParser();
        var xmlDoc = domParser.parseFromString(xmlData, "text/xml");
        var minLonLat = xmlDoc.getElementsByTagName("ows:WGS84BoundingBox")[0]
            .childNodes[1].textContent;
        var minLonLatSplit = minLonLat.split(" ");
        var minLon = minLonLatSplit[0];
        var minLat = minLonLatSplit[1];
        var maxLonLat = xmlDoc.getElementsByTagName("ows:WGS84BoundingBox")[0]
            .childNodes[3].textContent;
        var maxLonLatSplit = maxLonLat.split(" ");
        var maxLon = maxLonLatSplit[0];
        var maxLat = maxLonLatSplit[1];
        var bounds = {
            minLat: Number(minLat),
            minLon: Number(minLon),
            maxLat: Number(maxLat),
            maxLon: Number(maxLon),
        };
        return this.wrapperBounds(bounds);
    };
    OgcXmlParser.getOgcCenter = function (xmlData, service) {
        if (service === void 0) { service = OgcXmlParser.WMTS; }
        var bounds = {};
        if (OgcXmlParser.WMTS === service) {
            bounds = this.getWMTSBounds(xmlData);
        }
        else if (OgcXmlParser.WMS === service) {
            bounds = this.getWmsBounds(xmlData);
        }
        var minLatlon = this.createCoordinate(bounds.minLat, bounds.minLon);
        var maxLatlon = this.createCoordinate(bounds.maxLat, bounds.maxLon);
        var latLngBounds = L.latLngBounds(minLatlon, maxLatlon);
        var latLng = latLngBounds.getCenter();
        return [latLng.lat, latLng.lng];
    };
    OgcXmlParser.createCoordinate = function (lat, lon) {
        return L.latLng(lat, lon);
    };
    OgcXmlParser.WMTS = "WMTS";
    OgcXmlParser.WMS = "WMS";
    OgcXmlParser.WFS = "WFS";
    return OgcXmlParser;
}());
export default OgcXmlParser;
